#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
"""
Conveyor is responsible for moving or copying results from standard capability requests to the DA
QA area, called "delivery", and back, called "post QA".
"""
import argparse
import logging
import pathlib
import sys
from typing import Dict

from pycapo import CapoConfig

from .deliver import DeliveryConveyor
from .retrieve import RetrievalConveyor
from .solicitor import Solicitor

logger = logging.getLogger("conveyor")
logger.setLevel(logging.INFO)
logger.addHandler(logging.StreamHandler(sys.stdout))


def arg_parser() -> argparse.ArgumentParser:
    parser = argparse.ArgumentParser(
        description="Workspaces Standard Capability Postprocessing",
        formatter_class=argparse.RawTextHelpFormatter,
    )
    parser.add_argument(
        "--deliver",
        nargs=1,
        action="store",
        required=False,
        help="deliver standard calibration directories to analyst QA area",
    )
    parser.add_argument(
        "--retrieve",
        nargs=1,
        action="store",
        required=False,
        help="return standard calibration directories to original parent directory",
    )
    parser.add_argument(
        "--deliver-img",
        nargs=1,
        action="store",
        required=False,
        help="deliver standard image results to analyst image QA area",
    )
    parser.add_argument(
        "--retrieve-img",
        nargs=1,
        action="store",
        required=False,
        help="retrieve standard image results to original parent directory",
    )
    parser.add_argument(
        "--qa-fail",
        nargs=1,
        action="store",
        required=False,
        help="move results of failed QA to original parent directory",
    )

    return parser


def _get_settings(filename: str) -> Dict[str, str]:
    delivery_properties = CapoConfig().settings("edu.nrao.workspaces.DeliverySettings")
    processing_properties = CapoConfig().settings("edu.nrao.workspaces.ProcessingSettings")
    workflow_url = CapoConfig().settings("edu.nrao.workspaces.WorkflowSettings").serviceUrl
    external_capability_url = CapoConfig().settings("edu.nrao.workspaces.CapabilitySettings").externalServiceUrl

    delivery_area = delivery_properties.standardCalibrationDelivery
    weblog_cache_area = delivery_properties.cacheWeblogDirectory
    image_qa_area = delivery_properties.standardImageDelivery
    workspaces_lustre_root_dir = processing_properties.rootDirectory

    current_root_directory = str(pathlib.Path.cwd().parent)
    current_subdirectory = str(pathlib.Path.cwd().stem)
    destination_dir = Solicitor(filename=filename).solicit_delivery_location()
    destination_subdir = pathlib.Path(destination_dir).stem
    request_params = Solicitor(filename=filename).solicit_request_params()

    return {
        "qa_delivery_area": delivery_area,
        "weblog_cache_area": weblog_cache_area,
        "image_qa_area": image_qa_area,
        "workspaces_lustre_root_dir": workspaces_lustre_root_dir,
        "current_root_directory": current_root_directory,
        "current_subdirectory": current_subdirectory,
        "destination_dir": destination_dir,
        "destination_subdir": destination_subdir,
        "request_id": request_params["system_id"],
        "sdm_ids": request_params["sdm_ids"],
        "workflow_url": workflow_url,
        "capability_url": external_capability_url,
    }


def main():
    args = arg_parser().parse_args()
    action = None
    conveyor = None

    if args.deliver is not None:
        action = "Calibration Delivery"
        settings = _get_settings(args.deliver[0])
        conveyor = DeliveryConveyor(settings, action)
    elif args.retrieve is not None:
        action = "Calibration Retrieval"
        settings = _get_settings(args.retrieve[0])
        conveyor = RetrievalConveyor(settings, action)
    elif args.deliver_img is not None:
        action = "Image Delivery"
        settings = _get_settings(args.deliver_img[0])
        conveyor = DeliveryConveyor(settings, action)
    elif args.retrieve_img is not None:
        action = "Image Retrieval"
        settings = _get_settings(args.retrieve_img[0])
        conveyor = RetrievalConveyor(settings, action)
    elif args.qa_fail is not None:
        action = "QA Fail Calibration"
        settings = _get_settings(args.qa_fail[0])
        conveyor = RetrievalConveyor(settings, action)

    conveyor.convey()
    logger.info(f"Standard {action} is complete!")
